@@ -1,4 +1,4 @@
-# Copyright (c) 2011-2015 Bertrand Janin <b@janin.com>
+# Copyright (c) 2011-2017 Bertrand Janin <b@janin.com>
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -16,12 +16,15 @@ import csv
 import sys
 import os
 import subprocess
-import requests
 import tempfile
 import webbrowser
 import email.parser
+import warnings
 from collections import OrderedDict
 
+import requests
+from requests.packages.urllib3.exceptions import InsecureRequestWarning
+
 from cartman.compat import configparser
 from cartman import exceptions
 from cartman import ticket
@@ -36,11 +39,15 @@ CONFIG_LOCATIONS = [
 ]
 
 MIN_TRAC_VERSION = (0, 11)
-MAX_TRAC_VERSION = (1, 0)
+MAX_TRAC_VERSION = (1, 2)
 
+# List of all our accepted authentication types.  The values are authentication
+# classes used by the requests module.
 AUTH_TYPES = {
     "basic": requests.auth.HTTPBasicAuth,
     "digest": requests.auth.HTTPDigestAuth,
+    "acctmgr": None,
+    "none": None,
 }
 
 DEFAULT_TEMPLATE = """To:
@@ -91,7 +98,8 @@ class CartmanApp(object):
         self.session = requests.session()
 
         auth_class = AUTH_TYPES[self.auth_type]
-        self.session.auth = auth_class(self.username, self.password)
+        if auth_class:
+            self.session.auth = auth_class(self.username, self.password)
         self.session.verify = self.verify_ssl_cert
 
         func_name = "run_" + args.command
@@ -137,7 +145,7 @@ class CartmanApp(object):
             "verify_ssl_cert": "true",
         }
 
-        cp = configparser.RawConfigParser(defaults)
+        cp = configparser.SafeConfigParser(defaults)
         cp.read(CONFIG_LOCATIONS)
 
         if not cp.has_option(self.site, "base_url"):
@@ -145,10 +153,13 @@ class CartmanApp(object):
                                          "a base_url.".format(self.site))
 
         self.base_url = cp.get(self.site, "base_url").rstrip("/")
-        self.username = cp.get(self.site, "username")
-        self.password = cp.get(self.site, "password")
         self.verify_ssl_cert = cp.getboolean(self.site, "verify_ssl_cert")
 
+        # If you've decided not to verify your SSL certificate, you're on your
+        # own, there is no need to add more warnings.
+        if not self.verify_ssl_cert:
+            requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
+
         # load auth
         auth_type = cp.get(self.site, "auth_type")
         auth_type = auth_type.lower()
@@ -159,6 +170,21 @@ class CartmanApp(object):
             raise exceptions.ConfigError(msg)
         self.auth_type = auth_type
 
+        # On anonymous Trac systems, you may still specify a username, but you
+        # you are able to do some operations as anonymous.  For all other
+        # authentication types, username and password are mandatory.
+        if auth_type in ("none", "acctmgr"):
+            self.logged_in = True
+            try:
+                self.username = cp.get(self.site, "username")
+            except configparser.NoOptionError:
+                self.username = "anonymous"
+        else:
+            self.username = cp.get(self.site, "username")
+            self.password = os.environ.get('TRAC_PASSWORD', None)
+            if self.password is None:
+                self.password = cp.get(self.site, "password")
+
         if cp.has_option(self.site, "editor"):
             self.config_editor = cp.get(self.site, "editor")
         else:
@@ -305,6 +331,12 @@ class CartmanApp(object):
         # a different path to initiate authentication.
         r = self.get("/login", handle_errors=False)
 
+        if self.auth_type == "acctmgr":
+            r = self.post("/login", {
+                "user": self.username,
+                "password": self.password,
+            })
+
         if r.status_code not in (200, 302):
             msg = ("login failed on {} (bad user, password or auth type)"
                    .format(r.request.url))
@@ -468,10 +500,11 @@ class CartmanApp(object):
     #
 
     def run_change(self, ticket_id, *values):
-        """Make change to the given ticket_id. This command does not return
-        anything if successful.
+        """Make change to the given ticket_id.
 
-        TODO: support spanning an editor to change field values.
+        This command does not return anything if successful.
+
+        TODO: support spawning an editor to change field values.
 
         usage: cm change ticket_id field=value [field=value...]
 
